# -*- coding: utf-8 -*-

import sys
import os
import contextlib
import glob
import sqlite3

import unicode_csv

from sqlite_console import SQLiteConsole


class FlatQL(object):

    def __init__(self, database_path, **fmtparams):
        self._database_path = database_path
        self._sqlite_db = sqlite3.connect(':memory:')
        self._changed = False
        self._fmtparams = fmtparams

        FlatQL.load_database(
            self._sqlite_db,
            self._database_path,
            **self._fmtparams)

    def start_console(self):
        sqlite_console = SQLiteConsole(self._sqlite_db)
        sqlite_console.cmdloop("SQL Interactive Console")

        self._changed = sqlite_console.changed

    def query(self, sql_query, output=sys.stdout):
        with contextlib.closing(self._sqlite_db.cursor()) as c:
            c.execute(sql_query)

            if c.description is not None:
                headers = [col[0] for col in c.description]
                writer = unicode_csv.Writer(output)
                writer.writerow(headers)

                results = c.fetchall()
                writer.writerows(results)
            else:
                self._changed = True

    def __del__(self):
        if self._changed:
            FlatQL.save_database(
                self._sqlite_db,
                self._database_path,
                **self._fmtparams)
        self._sqlite_db.close()

    @staticmethod
    def load_database(database, database_path, **fmtparams):
        glob_string = os.path.join(database_path, '*.' + DEFAULT_SUFFIX)
        for table_path in glob.glob(glob_string):
            FlatQL.load_table(database, table_path, **fmtparams)

    @staticmethod
    def load_table(database, table_path, **fmtparams):
        table_name = os.path.splitext(os.path.basename(table_path))[0]

        with contextlib.closing(database.cursor()) as c, open(table_path, 'rU') as file_handle:
            csv_reader = unicode_csv.Reader(file_handle, **fmtparams)
            columns = [column.strip() for column in csv_reader.next()]

            c.execute(
                (u'CREATE TABLE IF NOT EXISTS "{table_name}"(' +
                 u', '.join([u'{}'] * len(columns)) + u');').format(
                    table_name=table_name,
                    *columns))

            query = (
                u'INSERT INTO "{table_name}"' +
                u'VALUES ({params})'
            ).format(
                table_name=table_name,
                params=u', '.join(u'?' * len(columns))
            )

            c.executemany(query, csv_reader)

        database.commit()

    @staticmethod
    def save_database(database_connection, database_path, **fmtparams):
        glob_string = os.path.join(database_path, '*.' + DEFAULT_SUFFIX)
        for table_path in glob.glob(glob_string):
            os.unlink(table_path)

        with contextlib.closing(database_connection.cursor()) as c:
            c.execute(
                u'SELECT name FROM sqlite_master' +
                u'WHERE' +
                u'type = \'table\' AND name NOT LIKE \'sqlite_%\';')
            write_tables = [row[0] for row in c.fetchall()]

        for write_table in write_tables:
            FlatQL.save_table(
                database_connection,
                database_path,
                write_table,
                **fmtparams)

    @staticmethod
    def save_table(
            database_connection,
            directory_path,
            table_name,
            **fmtparams):
        csv_file_path = os.path.join(directory_path, table_name + '.csv')
        with contextlib.closing(database_connection.cursor()) as c, open(csv_file_path, 'w') as file_handle:
            c.execute(
                u'SELECT sql FROM sqlite_master' +
                u'WHERE name = "{table_name}";'.format(
                    table_name=table_name))
            sql = c.fetchone()[0]
            columns_string = sql[sql.find("(") + 1:sql.find(")")]
            writer = unicode_csv.Writer(file_handle, **fmtparams)
            columns = [column.strip() for column in columns_string.split(',')]
            writer.writerow(columns)

            c.execute(
                u'SELECT * FROM {table_name}'.format(table_name=table_name))
            results = c.fetchall()
            writer.writerows(results)

def xxx():
  print 'xxx'
